@@ -1,10 +1,10 @@
 """The base Action class."""
 
-from typing import Any, BinaryIO, Dict, Iterable, Optional
+from typing import Any, BinaryIO, Iterable
 
 from bag.web.exceptions import Problem
-from kerno.pydantic import Pydantic, ZeroOrMore
-from pydantic import PyObject
+import colander as c
+from kerno.typing import DictStr
 
 from keepluggable.exceptions import FileNotAllowed
 from keepluggable.orchestrator import Orchestrator
@@ -24,28 +24,33 @@ class BaseFilesAction:
         """Instantiate an action for one request."""
         self.orchestrator = orchestrator
         self.namespace = namespace
-        self.config = orchestrator.action_config
+        self.config: DictStr = orchestrator.action_config
 
-    class Config(Pydantic):
+    class Config(c.Schema):
         """Validated configuration for BaseFilesAction.
 
-        - ``max_file_size`` (int): the maximum file length, in bytes, that
-          can be uploaded. When zero, the system does not have a maximum size.
+        - ``max_file_size`` (int): the maximum file length, in bytes, that can be
+          uploaded. When zero, the system does not have a maximum size.  Default: 0.
         - ``allow_empty_files`` (boolean): whether to allow zero-length
-          files to be uploaded.
+          files to be uploaded.  Default: false.
         - ``cls_update_metadata_schema`` (dotted resource spec):
           Colander schema that validates metadata being updated.
           Without it, no validation is done, which is unsafe.
           So it is recommended that you implement a schema.
         """
 
-        max_file_size: ZeroOrMore = 0
-        allow_empty_files: bool = False
-        cls_update_metadata_schema: Optional[PyObject] = None
+        max_file_size = c.SchemaNode(c.Int(), validator=c.Range(min=0), missing=0)
+        allow_empty_files = c.SchemaNode(c.Bool(), missing=False)
+        cls_update_metadata_schema = c.SchemaNode(
+            c.GlobalObject(package=None), missing=None
+        )
 
-    def store_original_file(
-        self, bytes_io: BinaryIO, **metadata
-    ) -> Dict[str, Any]:
+    @classmethod
+    def get_config(cls, settings: DictStr) -> DictStr:
+        """Stuff called by the orchestrator at startup."""
+        return cls.Config().deserialize(settings)
+
+    def store_original_file(self, bytes_io: BinaryIO, repo: Any, **metadata) -> DictStr:
         """Point of entry into the workflow of storing a file.
 
         You can override this method in subclasses to change the steps
@@ -64,17 +69,19 @@ class BaseFilesAction:
 
         # In the case of images, keepluggable can be configured to not store
         # the original, but it still must be checked for duplicates
-        self._check_for_existing_file(bytes_io=bytes_io, metadata=metadata)
+        self._check_for_existing_file(bytes_io=bytes_io, metadata=metadata, repo=repo)
 
         # Hook for subclasses to allow or forbid storing this file
         # by raising FileNotAllowed
         self._allow_storage_of(bytes_io=bytes_io, metadata=metadata)
 
-        self._store_versions(bytes_io=bytes_io, metadata=metadata)
+        self._store_versions(bytes_io=bytes_io, metadata=metadata, repo=repo)
         return self._complement(metadata)
 
     def _compute_file_metadata(
-        self, bytes_io: BinaryIO, metadata: Dict[str, Any],
+        self,
+        bytes_io: BinaryIO,
+        metadata: DictStr,
     ) -> None:
         """Override this method in subclasses to populate the file metadata."""
         self._guess_mime_type(bytes_io, metadata)
@@ -82,7 +89,9 @@ class BaseFilesAction:
         self._compute_md5(bytes_io, metadata)
 
     def _guess_mime_type(
-        self, bytes_io: BinaryIO, metadata: Dict[str, Any],
+        self,
+        bytes_io: BinaryIO,
+        metadata: DictStr,
     ) -> None:
         """Discover the MIME type from the uploaded file extension.
 
@@ -93,19 +102,29 @@ class BaseFilesAction:
         """
         from mimetypes import guess_type
 
-        typ = guess_type(metadata["file_name"])[0]
+        file_name = metadata["file_name"]
+        typ = None
+        if file_name.endswith((".HEIC", ".HEIF", ".heic", ".heif")):
+            typ = "image/heic"  # because Ubuntu 2020 is lacking this MIME type
+        else:
+            typ = guess_type(file_name)[0]  # might be None again
         if typ:
             metadata["mime_type"] = typ
+        # else keep the value provided by the browser.
 
     def _compute_length(
-        self, bytes_io: BinaryIO, metadata: Dict[str, Any],
+        self,
+        bytes_io: BinaryIO,
+        metadata: DictStr,
     ) -> None:
         from bag.streams import get_file_size
 
         metadata["length"] = get_file_size(bytes_io)
 
     def _compute_md5(
-        self, bytes_io: BinaryIO, metadata: Dict[str, Any],
+        self,
+        bytes_io: BinaryIO,
+        metadata: DictStr,
     ) -> None:
         from hashlib import md5
 
@@ -124,20 +143,23 @@ class BaseFilesAction:
         if previous_length is None:
             metadata["length"] = the_length
         else:
-            assert previous_length == the_length, (
-                "Bug? File lengths {}, {} "
-                "don't match.".format(previous_length, the_length)
+            assert (
+                previous_length == the_length
+            ), "Bug? File lengths {}, {} " "don't match.".format(
+                previous_length, the_length
             )
         bytes_io.seek(0)  # ...so it can be read again
 
     def _allow_storage_of(
-        self, bytes_io: BinaryIO, metadata: Dict[str, Any],
+        self,
+        bytes_io: BinaryIO,
+        metadata: DictStr,
     ) -> None:
         """Override this method if you wish to abort storing some files.
 
         To abort, raise FileNotAllowed with a message explaining why.
         """
-        maximum = self.config.max_file_size
+        maximum = self.config["max_file_size"]
         if maximum and metadata["length"] > maximum:
             raise FileNotAllowed(
                 "The file is {} KB long and the maximum is {} KB.".format(
@@ -145,42 +167,52 @@ class BaseFilesAction:
                 )
             )
 
-        if not self.config.allow_empty_files and metadata["length"] == 0:
+        if not self.config["allow_empty_files"] and metadata["length"] == 0:
             raise FileNotAllowed("The file is empty.")
 
     def _store_versions(
-        self, bytes_io: BinaryIO, metadata: Dict[str, Any],
+        self,
+        bytes_io: BinaryIO,
+        metadata: DictStr,
+        repo: Any,
     ) -> None:
         """In this base class, just call _store_file().
 
         But most subclasses will have a complex workflow for storing versions.
         """
         metadata["versions"] = []
-        self._store_file(bytes_io, metadata)
+        self._store_file(bytes_io, metadata, repo)
 
     def _check_for_existing_file(
-        self, bytes_io: BinaryIO, metadata: Dict[str, Any],
+        self,
+        bytes_io: BinaryIO,
+        metadata: DictStr,
+        repo: Any,
     ) -> None:
         # Hook for subclasses to deviate if the file already exists
         if hasattr(self, "_handle_upload_of_existing_file"):
             existing = self._file_already_exists(metadata)
-            # print(existing, metadata['version'], metadata)
-            # import ipdb; ipdb.set_trace() # TODO Remove debug
             if existing:
-                self._handle_upload_of_existing_file(  # type: ignore
-                    bytes_io=bytes_io, metadata=metadata, existing=existing,
+                self._handle_upload_of_existing_file(  # type: ignore[attr-defined]
+                    bytes_io=bytes_io,
+                    metadata=metadata,
+                    existing=existing,
+                    repo=repo,
                 )
                 # TODO We should roll back any versions stored in AWS
                 # In fact, we should validate everything before sending any
 
-    def _file_already_exists(self, metadata: Dict[str, Any]) -> Dict[str, Any]:
+    def _file_already_exists(self, metadata: DictStr) -> DictStr:
         """Return existing file with the same md5 in the namespace, or None."""
-        return self.orchestrator.storage_metadata.get(
+        return self.orchestrator.storage_metadata.get_entity(
             namespace=self.namespace, key=metadata["md5"]
         )
 
     def _store_file(
-        self, bytes_io: BinaryIO, metadata: Dict[str, Any],
+        self,
+        bytes_io: BinaryIO,
+        metadata: DictStr,
+        repo: Any,
     ) -> None:
         """Save the payload and the metadata on the 2 storage backends.
 
@@ -189,12 +221,10 @@ class BaseFilesAction:
 
         But first we check for duplicates of the file being stored.
         """
-        self._check_for_existing_file(bytes_io=bytes_io, metadata=metadata)
+        self._check_for_existing_file(bytes_io=bytes_io, metadata=metadata, repo=repo)
 
         storage_file = self.orchestrator.storage_file
-        storage_file.put(
-            namespace=self.namespace, metadata=metadata, bytes_io=bytes_io
-        )
+        storage_file.put(namespace=self.namespace, metadata=metadata, bytes_io=bytes_io)
 
         try:
             self._store_metadata(bytes_io, metadata)
@@ -203,7 +233,9 @@ class BaseFilesAction:
             raise
 
     def _store_metadata(
-        self, bytes_io: BinaryIO, metadata: Dict[str, Any],
+        self,
+        bytes_io: BinaryIO,
+        metadata: DictStr,
     ) -> None:
         self.orchestrator.storage_metadata.put(
             namespace=self.namespace, metadata=metadata
@@ -229,20 +261,16 @@ class BaseFilesAction:
         for metadata in metadatas:
             sm.delete(self.namespace, metadata["md5"])
 
-    def gen_originals(self, filters=None) -> Iterable[Dict[str, Any]]:
+    def gen_originals(self, filters=None) -> Iterable[DictStr]:
         """Yield the original files in this namespace.
 
         ...optionally with further filters.
         """
         # This implementation queries the DB once rather than thousands:
         universe = list(
-            self.orchestrator.storage_metadata.gen_all(
-                self.namespace, filters=filters
-            )
+            self.orchestrator.storage_metadata.gen_all(self.namespace, filters=filters)
         )
-        originals = {
-            f["id"]: f for f in universe if f["version"] == "original"
-        }
+        originals = {f["id"]: f for f in universe if f["version"] == "original"}
         for f in originals.values():
             f["versions"] = []
         for adict in universe:
@@ -257,7 +285,7 @@ class BaseFilesAction:
         for fil in originals.values():
             yield self._complement(fil)
 
-    def _complement(self, metadata: Dict[str, Any]) -> Dict[str, Any]:
+    def _complement(self, metadata: DictStr) -> DictStr:
         """Add the links for downloading the original file and its versions."""
         url = self.orchestrator.storage_file.get_url
 
@@ -270,17 +298,20 @@ class BaseFilesAction:
         return metadata
 
     def _validate_metadata_for_updating(
-        self, adict: Dict[str, Any],
-    ) -> Dict[str, Any]:
-        cls_update_metadata_schema = self.config.cls_update_metadata_schema
+        self,
+        adict: DictStr,
+    ) -> DictStr:
+        cls_update_metadata_schema = self.config["cls_update_metadata_schema"]
         if cls_update_metadata_schema is not None:
             schema = cls_update_metadata_schema()
             adict = schema.deserialize(adict)
         return adict
 
     def update_metadata(
-        self, id: int, adict: Dict[str, Any],
-    ) -> Dict[str, Any]:
+        self,
+        id: int,
+        adict: DictStr,
+    ) -> DictStr:
         """Replace the metadata for key *id* with *adict*."""
         return self._complement(
             self.orchestrator.storage_metadata.update(
