@@ -4,43 +4,49 @@ from copy import copy
 from io import BytesIO
 from typing import Any, BinaryIO, Dict, List, Union
 
+from bag.text import strip_lower_preparer, strip_preparer
+import colander as c
+
 # import imghdr  # imghdr.what(file)
-from kerno.pydantic import Pydantic, ReqStr
-from pydantic import PositiveInt, validator
+from kerno.typing import DictStr
 from PIL import Image, ExifTags
+from pillow_heif import register_heif_opener
 
 from keepluggable.actions import BaseFilesAction
 from keepluggable.exceptions import FileNotAllowed
+from keepluggable.orchestrator import Orchestrator
+
+register_heif_opener()  # and now Pillow can read the HEIC format.
 
 
-class ImageVersionConfig(Pydantic):
+def _image_format_validator(node, value: str):
+    if value not in ("png", "jpeg", "gif"):
+        raise c.Invalid(node, f"Unknown image format: {value}")
+    return value
+
+
+class ImageVersionConfig(c.MappingSchema):
     """A part of the configuration."""
 
-    format: ReqStr
-    width: PositiveInt
-    height: PositiveInt
-    name: ReqStr
-
-    @validator("format")
-    def validate_format(cls, value: str) -> str:
-        """Convert format to lower case."""
-        value = value.lower()
-        if value not in ("png", "jpeg", "gif"):
-            raise ValueError(f"Unknown format: {value}")
-        return value
+    format = c.SchemaNode(
+        c.String(), preparer=strip_lower_preparer, validator=_image_format_validator
+    )
+    height = c.SchemaNode(c.Int(), validator=c.Range(min=1))
+    width = c.SchemaNode(c.Int(), validator=c.Range(min=1))
+    name = c.SchemaNode(c.String(), preparer=strip_preparer, validator=c.Length(min=1))
 
     @classmethod
-    def from_str(cls, line: str) -> "ImageVersionConfig":
-        """Instantiate from a configuration line."""
+    def from_str(cls, line: str) -> DictStr:
+        """From a configuration line, return a config dict."""
         parts = line.split()
-        assert (
-            len(parts) == 4
-        ), f'The configuration line "{line}" should have 4 parts'
-        return cls(
-            format=parts[0],
-            width=parts[1],
-            height=parts[2],
-            name=parts[3],
+        assert len(parts) == 4, f'The configuration line "{line}" should have 4 parts'
+        return cls().deserialize(
+            {
+                "format": parts[0],
+                "width": parts[1],
+                "height": parts[2],
+                "name": parts[3],
+            }
         )
 
 
@@ -65,7 +71,7 @@ class ImageAction(BaseFilesAction):
         sudo ln -s /usr/lib/x86_64-linux-gnu/libjpeg.so /usr/lib
         sudo ln -s /usr/lib/x86_64-linux-gnu/libfreetype.so /usr/lib
         sudo ln -s /usr/lib/x86_64-linux-gnu/libz.so /usr/lib
-        easy_install -UZ Pillow
+        pip install Pillow
 
     Pay attention to the desired supported formats near the end of the output::
 
@@ -113,30 +119,34 @@ class ImageAction(BaseFilesAction):
     class Config(BaseFilesAction.Config):
         """Validated configuration for ``ImageAction``."""
 
-        upload_must_be_img: bool = False
-        store_original: bool = True
-        versions_quality: int = 90
-        versions: List[ImageVersionConfig]
+        upload_must_be_img = c.SchemaNode(c.Bool(), missing=False)
+        store_original = c.SchemaNode(c.Bool(), missing=True)
+        versions_quality = c.SchemaNode(c.Int(), missing=90)
 
-        @validator("versions", pre=True, each_item=False)
-        def validate_versions(
-            cls,
-            value: Union[List[ImageVersionConfig], str],
-        ) -> List[ImageVersionConfig]:
-            """Convert the configuration string into validated objects."""
-            if not isinstance(value, str):
-                return value
+    @classmethod
+    def get_config(cls, settings: DictStr) -> DictStr:
+        """Image versions are a complex string in configuration; parse them.
 
-            # Convert str to ImageVersionConfig
-            versions = []
-            for line in value.split("\n"):
-                line = line.strip()
-                if not line:  # Ignore an empty line
-                    continue
-                versions.append(ImageVersionConfig.from_str(line))
-            # We want to process from smaller to bigger:
-            versions.sort(key=lambda d: d.width)
-            return versions
+        This gets called by the orchestrator at startup.
+        Return the entire action configuration dictionary.
+        """
+        value = settings["versions"]
+        if not isinstance(value, str):
+            return value
+
+        # Convert str to validated dict
+        versions: List[DictStr] = []
+        for line in value.split("\n"):
+            line = line.strip()
+            if not line:  # Ignore an empty line
+                continue
+            versions.append(ImageVersionConfig.from_str(line))
+        # We want to process image versions from smaller to bigger:
+        versions.sort(key=lambda d: d["width"])
+
+        config: DictStr = cls.Config().deserialize(settings)
+        config["versions"] = versions
+        return config
 
     def _img_from_stream(
         self,
@@ -177,17 +187,18 @@ class ImageAction(BaseFilesAction):
         self,
         bytes_io: BinaryIO,
         metadata: Dict[str, Any],
+        repo: Any,
     ) -> None:
         # We override this method to deal with images.
         is_image = metadata["mime_type"].startswith("image")
         if not is_image:
-            if self.config.upload_must_be_img:
+            if self.config["upload_must_be_img"]:
                 raise FileNotAllowed(
                     'The file name "{}" lacks a supported image extension, '
                     "so it was not stored.".format(metadata["file_name"])
                 )
             else:
-                super()._store_versions(bytes_io, metadata)
+                super()._store_versions(bytes_io, metadata, repo)
                 return
 
         # # If you need to load the image after verify(), must reopen it
@@ -196,13 +207,13 @@ class ImageAction(BaseFilesAction):
         original = self._rotate_exif_orientation(original)
 
         # Probably don't need to verify() the image since we are loading it
-        # original.verify()  # TODO What does this raise?
+        # original.verify()  # What does this raise?
         self._copy_img(original, metadata)  # Try to raise before storing
 
         #  No exceptions were raised,  so store the original file
         metadata["image_width"], metadata["image_height"] = original.size
-        if self.config.store_original:  # Optionally store original payload
-            self._store_file(bytes_io, metadata)
+        if self.config["store_original"]:  # Optionally store original payload
+            self._store_file(bytes_io, metadata, repo)
         else:  # Always store original metadata
             self._store_metadata(bytes_io, metadata)
 
@@ -212,13 +223,13 @@ class ImageAction(BaseFilesAction):
         largest_version_created_so_far = 0
         original_area = original.size[0] * original.size[1]
         new_versions = []
-        for version_config in self.config.versions:
-            current_area = version_config.width * version_config.height
+        for version_config in self.config["versions"]:
+            current_area = version_config["width"] * version_config["height"]
             if largest_version_created_so_far <= original_area:
                 # Do it
                 new_versions.append(
                     self._store_img_version(  # may raise
-                        original, metadata, version_config
+                        original, metadata, version_config, repo
                     )
                 )
                 largest_version_created_so_far = current_area
@@ -229,16 +240,17 @@ class ImageAction(BaseFilesAction):
         original: Image,
         original_metadata: Dict[str, Any],
         version_config: ImageVersionConfig,
+        repo: Any,
     ) -> Dict[str, Any]:
         metadata = copy(original_metadata)
-        metadata["version"] = version_config.name
+        metadata["version"] = version_config["name"]
         metadata["original_id"] = original_metadata["id"]
         del metadata["id"]
 
         img = self._convert_img(original, metadata, version_config)
 
         # Store the new metadata and the new payload
-        self._store_file(img.stream, metadata)
+        self._store_file(img.stream, metadata, repo)
 
         return metadata
 
@@ -254,33 +266,31 @@ class ImageAction(BaseFilesAction):
         except OSError:
             raise FileNotAllowed(
                 'Unable to store the image "{}" because '
-                "the server is unable to convert it.".format(
-                    metadata["file_name"]
-                )
+                "the server is unable to convert it.".format(metadata["file_name"])
             )
 
     def _convert_img(
         self,
         original: Image,
         metadata: Dict[str, Any],
-        version_config: ImageVersionConfig,
+        version_config: DictStr,
         resample=Image.LANCZOS,
     ) -> Image:
         """Return a new image, converted from ``original``.
 
         Do it using ``version_config`` and setting ``metadata``.
         """
-        fmt = version_config.format
+        fmt = version_config["format"]
 
         # Resize, keeping the aspect ratio:
         img = self._copy_img(original, metadata, alpha=fmt != "jpeg")
-        img.thumbnail((version_config.width, version_config.height), resample)
+        img.thumbnail((version_config["width"], version_config["height"]), resample)
 
         stream = BytesIO()
         img.save(
             stream,
             format=fmt.upper(),
-            quality=self.config.versions_quality,
+            quality=self.config["versions_quality"],
             optimize=1,
         )
         img.stream = stream  # so we can recover it elsewhere
@@ -297,6 +307,6 @@ class ImageAction(BaseFilesAction):
         """Omit the main *href* if we are not storing original images."""
         metadata = super()._complement(metadata)
         # Add main *href* if we are storing original images or if not image
-        if metadata.get("image_width") and not self.config.store_original:
+        if metadata.get("image_width") and not self.config["store_original"]:
             del metadata["href"]
         return metadata
