@@ -3,33 +3,34 @@
 from os import PathLike
 from typing import Any, Dict, Union
 
-from pydantic import PyObject, validator
+from bag.text import strip_preparer
+import colander as c
+from kerno.typing import DictStr
 import reg
 
-from kerno.pydantic import Pydantic, ReqStr
 
-
-class Configuration(Pydantic):
+class ConfigurationSchema(c.Schema):
     """Validated configuration of a keepluggable instance.
 
     ``settings`` should contain only the relevant section of an INI file.
     """
 
-    name: ReqStr
-    settings: Dict[str, Any]
-    cls_storage_file: PyObject
-    cls_storage_metadata: PyObject
-    cls_action: PyObject
-
-    @validator("cls_storage_file")
-    def _validate_storage_file(cls, value):
+    @staticmethod
+    def _validate_storage_file(node, value):
         from .storage_file import BasePayloadStorage as BPS
 
         if not issubclass(value, BPS) or value is BPS:
-            raise ValueError(
-                "*cls_storage_file* must be a subclass of BasePayloadStorage."
+            raise c.Invalid(
+                node, "*cls_storage_file* must be a subclass of BasePayloadStorage."
             )
-        return value
+
+    name = c.SchemaNode(c.Str(), preparer=strip_preparer, validator=c.Length(min=1))
+    cls_action = c.SchemaNode(c.GlobalObject(package=None))
+    cls_storage_metadata = c.SchemaNode(c.GlobalObject(package=None))
+    cls_storage_file = c.SchemaNode(
+        c.GlobalObject(package=None),
+        validator=_validate_storage_file.__func__,  # type: ignore[attr-defined]
+    )
 
 
 class Orchestrator:
@@ -45,21 +46,22 @@ class Orchestrator:
 
     instances: Dict[str, "Orchestrator"] = {}
 
-    def __init__(self, config: Configuration) -> None:
-        """Instantiate from a validated configuration object."""
+    def __init__(self, config: DictStr) -> None:
+        """Instantiate from a validated configuration dictionary."""
         self.config = config
-        self.storage_file = config.cls_storage_file(self)  # type: ignore
-        self.storage_metadata = config.cls_storage_metadata(  # type: ignore
-            self
-        )
-        Orchestrator.instances[config.name] = self
-        self.action_config = config.cls_action.Config(  # type: ignore
-            **config.settings
+        self.storage_file = config["cls_storage_file"](self)
+        self.storage_metadata = config["cls_storage_metadata"](self)
+        Orchestrator.instances[config["name"]] = self
+        self.action_config: DictStr = config["cls_action"].get_config(
+            config["settings"]
         )
 
     @classmethod
     def from_ini(
-        cls, name: str, *paths: Union[str, PathLike], encoding: str = "utf-8",
+        cls,
+        name: str,
+        *paths: Union[str, PathLike],
+        encoding: str = "utf-8",
     ) -> "Orchestrator":
         """Read one or more INI files and return an Orchestrator instance."""
         from configparser import ConfigParser
@@ -67,28 +69,29 @@ class Orchestrator:
         parser = ConfigParser()
         parser.read(paths, encoding=encoding)
         section = parser["keepluggable " + name]
-        config = Configuration(
-            name=name,
-            settings=section,
-            cls_storage_file=section["cls_storage_file"],
-            cls_storage_metadata=section["cls_storage_metadata"],
-            cls_action=section["cls_action"],
+        config: DictStr = ConfigurationSchema().deserialize(
+            {
+                "name": name,
+                "cls_storage_file": section["cls_storage_file"],
+                "cls_storage_metadata": section["cls_storage_metadata"],
+                "cls_action": section["cls_action"],
+            }
         )
+        config["settings"] = section  # each component takes its own settings from here
         return cls(config)
 
     def get_action(self, namespace: str) -> Any:
         """Conveniently instantiate the configured action class."""
-        return self.config.cls_action(self, namespace)  # type: ignore
+        return self.config["cls_action"](self, namespace)
 
-    def __repr__(self):
-        return f'<Orchestrator "{self.config.name}">'
+    def __repr__(self) -> str:
+        return f'<Orchestrator "{self.config["name"]}">'
 
 
 @reg.dispatch(  # Dispatch on the value of *name*.
     reg.match_key("name", lambda name, namespace: name)
 )
-# Cannot type-annotate this function, Reg 0.11 does not support it
-def get_middle_path(name, namespace):
+def get_middle_path(name: str, namespace: Any) -> str:
     """Return the path between bucket and file name.
 
     By default this simply returns the ``namespace``. This means the
