@@ -8,11 +8,12 @@ from typing import Any, BinaryIO, Callable, Dict, Iterable, Sequence
 from urllib.parse import quote
 
 from bag import dict_subset
-from kerno.pydantic import Pydantic, ReqStr
+from bag.text import strip_preparer
+import colander as c
 
 # http://botocore.readthedocs.org/en/latest/
 from botocore.exceptions import ClientError
-from boto3.session import Session  # easy_install -UZ boto3
+from boto3.session import Session
 
 from keepluggable.orchestrator import get_middle_path, Orchestrator
 from keepluggable.storage_file import BasePayloadStorage, get_extension
@@ -20,7 +21,7 @@ from keepluggable.storage_file import BasePayloadStorage, get_extension
 DAY = 60 * 60 * 24  # seconds
 
 
-class S3Config(Pydantic):
+class S3ConfigSchema(c.Schema):
     """Configuration settings for AmazonS3Storage.
 
     - ``s3_access_key_id``: part of your Amazon credentials
@@ -29,10 +30,18 @@ class S3Config(Pydantic):
     - ``s3_bucket``: name of the bucket in which to store objects.
     """
 
-    s3_access_key_id: ReqStr
-    s3_access_key_secret: ReqStr
-    s3_region_name: ReqStr
-    s3_bucket: ReqStr
+    s3_access_key_id = c.SchemaNode(
+        c.Str(), preparer=strip_preparer, validator=c.Length(min=1)
+    )
+    s3_access_key_secret = c.SchemaNode(
+        c.Str(), preparer=strip_preparer, validator=c.Length(min=8)
+    )
+    s3_region_name = c.SchemaNode(
+        c.Str(), preparer=strip_preparer, validator=c.Length(min=4)
+    )
+    s3_bucket = c.SchemaNode(
+        c.Str(), preparer=strip_preparer, validator=c.Length(min=1)
+    )
 
 
 class AmazonS3Storage(BasePayloadStorage):
@@ -46,26 +55,24 @@ class AmazonS3Storage(BasePayloadStorage):
     def __init__(self, orchestrator: Orchestrator) -> None:
         """Read settings and instantiate an S3 Session."""
         super().__init__(orchestrator)
-        self.config = S3Config(**self.orchestrator.config.settings)
+        self.config = S3ConfigSchema().deserialize(self.orchestrator.config["settings"])
         session = Session(
-            aws_access_key_id=self.config.s3_access_key_id,
-            aws_secret_access_key=self.config.s3_access_key_secret,
-            region_name=self.config.s3_region_name,
+            aws_access_key_id=self.config["s3_access_key_id"],
+            aws_secret_access_key=self.config["s3_access_key_secret"],
+            region_name=self.config["s3_region_name"],
         )
         self.s3 = session.resource("s3")
         self._set_bucket()
 
     def _set_bucket(self, bucket_name=None):
-        self.bucket_name = bucket_name or self.config.s3_bucket
+        self.bucket_name = bucket_name or self.config["s3_bucket"]
         self.bucket = self.s3.Bucket(self.bucket_name)
 
     SEP = "/"
 
     def _get_path(self, namespace: str, metadata: Dict[str, Any]) -> str:
         return (
-            get_middle_path(
-                name=self.orchestrator.config.name, namespace=namespace
-            )
+            get_middle_path(name=self.orchestrator.config["name"], namespace=namespace)
             + self.SEP
             + self._get_filename(metadata)
         )
@@ -86,7 +93,10 @@ class AmazonS3Storage(BasePayloadStorage):
             return adict["Body"]
 
     def put(
-        self, namespace: str, metadata: Dict[str, Any], bytes_io: BinaryIO,
+        self,
+        namespace: str,
+        metadata: Dict[str, Any],
+        bytes_io: BinaryIO,
     ) -> None:
         """Store a file."""
         subset = dict_subset(
@@ -104,19 +114,18 @@ class AmazonS3Storage(BasePayloadStorage):
         if hasattr(bytes_io, "seekable") and bytes_io.seekable():
             bytes_io.seek(0)
 
-        # When botocore.response.StreamingBody is passed in as bytes_io,
-        # the bucket.put_object() call below fails with
-        # "AttributeError: 'StreamingBody' object has no attribute 'tell'"
-        # so we have to read the stream, getting the bytes:
-        if not hasattr(bytes_io, "tell"):
-            bytes_io = bytes_io.read()  # type: ignore
+        byts: bytes = (
+            bytes_io.read()
+            if hasattr(bytes_io, "tell")
+            else bytes_io  # type: ignore[assignment]
+        )
 
         result = self.bucket.put_object(
             Key=self._get_path(namespace, metadata),
             # done automatically by botocore:  ContentMD5=encoded_md5,
             ContentType=metadata["mime_type"],
             ContentLength=metadata["length"],
-            Body=bytes_io,
+            Body=byts,
             Metadata=subset,
         )
         # print(result)
@@ -144,7 +153,7 @@ class AmazonS3Storage(BasePayloadStorage):
             seconds, self.bucket_name, composite
         ).encode("ascii")
         digest = hmac.new(
-            self.config.s3_access_key_secret.encode("ascii"), to_sign, sha1
+            self.config["s3_access_key_secret"].encode("ascii"), to_sign, sha1
         ).digest()
         return (
             "{scheme}{bucket}.s3.amazonaws.com/{key}?AWSAccessKeyId="
@@ -152,14 +161,16 @@ class AmazonS3Storage(BasePayloadStorage):
                 scheme="https://" if https else "http://",
                 bucket=self.bucket_name,
                 key=composite,
-                access_key_id=self.config.s3_access_key_id,
+                access_key_id=self.config["s3_access_key_id"],
                 seconds=seconds,
                 signature=quote(base64.encodebytes(digest).strip()),
             )
         )
 
     def delete(
-        self, namespace: str, metadatas: Sequence[Dict[str, Any]],
+        self,
+        namespace: str,
+        metadatas: Sequence[Dict[str, Any]],
     ) -> Any:
         """Delete up to 1000 files."""
         number = len(metadatas)
@@ -213,7 +224,7 @@ class AmazonS3Power(AmazonS3Storage):
     def gen_paths(self, namespace: str) -> Iterable[str]:
         """Generate the paths in a namespace. Too costly -- avoid."""
         prefix = get_middle_path(
-            name=self.orchestrator.config.name, namespace=namespace
+            name=self.orchestrator.config["name"], namespace=namespace
         )
         for o in self.bucket.objects.all():
             composite = o.key
@@ -227,11 +238,7 @@ class AmazonS3Power(AmazonS3Storage):
         """
         # TODO Work around S3 limitation of 1000 objects per request
         self.bucket.delete_objects(
-            Delete={
-                "Objects": [
-                    {"Key": path} for path in self.gen_paths(namespace)
-                ]
-            }
+            Delete={"Objects": [{"Key": path} for path in self.gen_paths(namespace)]}
         )
 
     def empty_bucket(self):
@@ -281,14 +288,8 @@ class AmazonS3Power(AmazonS3Storage):
         print("   Retrieving existing keys in target {}".format(self.bucket))
 
         # TODO For a really big bucket we might need to use a database:
-        existing = [
-            old_path_from_new_path(fil.key) for fil in new_objects_collection
-        ]
-        print(
-            "   There are {}. Migrating remaining files...".format(
-                len(existing)
-            )
-        )
+        existing = [old_path_from_new_path(fil.key) for fil in new_objects_collection]
+        print("   There are {}. Migrating remaining files...".format(len(existing)))
 
         for index, summary in enumerate(old.objects.all(), 1):
             if index < skip_the_first_n:
@@ -311,14 +312,12 @@ class AmazonS3Power(AmazonS3Storage):
             # Ignore image versions found in "discard_img_sizes"
             version = obj.metadata.get("version")
             if version in discard_img_sizes:
-                print(
-                    "   {}. Skipping unwanted version: {}".format(index, obj)
-                )
+                print("   {}. Skipping unwanted version: {}".format(index, obj))
                 continue
 
             new_key = (
                 get_middle_path(
-                    name=self.orchestrator.config.name, namespace=namespace
+                    name=self.orchestrator.config["name"], namespace=namespace
                 )
                 + self.SEP
                 + md5
